header{
	package es.us.isa.gdl.choco.parser;
	import java.util.*;	
	import choco.Choco;
	import choco.kernel.model.constraints.Constraint;
	import choco.kernel.model.variables.Variable;
	import choco.kernel.model.variables.integer.IntegerVariable;
	import choco.kernel.model.variables.integer.IntegerConstantVariable;
	import choco.kernel.model.variables.real.RealVariable;
	import choco.kernel.model.variables.integer.IntegerExpressionVariable;
	import choco.kernel.model.variables.real.RealExpressionVariable;
	import es.us.isa.gdl.choco.*;
}

class ChocoTreeParser extends TreeParser;

options{
	importVocab = ChocoAnasint;
}

{
	Map<String,Variable> variables;
	
	Collection<String> unknownVars;
	
	GDAChocoAnalyzer choco;
	
	protected IntegerVariable getVariable(AST ident){
		IntegerVariable res = null;
		String s = ident.getText();
		Variable v = variables.get(s);
		if (v != null && (v instanceof IntegerVariable)){
			res = (IntegerVariable)v;
		}
		else{
			unknownVars.add(s);
		}
		return res;
	}	
	
	protected IntegerConstantVariable toInt(AST t){
		String s = t.getText();
		int i = Integer.parseInt(s);
		IntegerConstantVariable res = Choco.constant(i);
		return res;
	}
	
	protected IntegerConstantVariable toBoolean(AST t){
		String s = t.getText();
		if (s.equals("FALSE")){
			return Choco.constant(0);	
		}	
		else{
			return Choco.constant(1);
		}
	}
	
	protected IntegerConstantVariable toConstant(AST t){
		String aux = t.getText();
		String id = removeQuotes(aux);
		Integer i = choco.id2Constant(id);
		return Choco.constant(i);
	}
	
	protected String removeQuotes(String s){
		String res = s.substring(1,s.length());
		return res;
	}
		
}

entrada[Map<String,Variable> vars, GDAChocoAnalyzer a] returns [ParsingResult res = null]
{Constraint aux = null;}:
{variables = vars;
choco = a;
unknownVars = new LinkedList<String>();} 
aux = expresion_logica
{res = new ParsingResult(aux,unknownVars);};

expresion_logica returns [Constraint res = null]
{Constraint c1 = null, c2 = null;}: 
	   #(IFF c1 = expresion_logica c2 = expresion_logica
	   	{res = Choco.ifOnlyIf(c1,c2);})
	 | #(IMPLIES c1 = expresion_logica c2 = expresion_logica
	 	{res = Choco.implies(c1,c2);})
	 | #(OR c1 = expresion_logica c2 = expresion_logica
	 	{res = Choco.or(c1,c2);})
	 | #(AND c1 = expresion_logica c2 = expresion_logica
	 	{res = Choco.and(c1,c2);})
	 | #(NOT c1 = expresion_logica {res = Choco.not(c1);})
	 | res = expresion_relacional;
	 
expresion_relacional returns [Constraint res = null]
{IntegerExpressionVariable v1 = null, v2 = null;}: 
	   #(MAYOR v1 = expresion_aritmetica v2 = expresion_aritmetica
	 	{res = Choco.gt(v1,v2);})
	 | #(MENOR v1 = expresion_aritmetica v2 = expresion_aritmetica
	   	{res = Choco.lt(v1,v2);})
	 | #(MAYOR_IGUAL v1 = expresion_aritmetica v2 = expresion_aritmetica
	 	{res = Choco.geq(v1,v2);})
	 | #(MENOR_IGUAL v1 = expresion_aritmetica v2 = expresion_aritmetica
	 	{res = Choco.leq(v1,v2);})
	 | #(IGUAL v1 = expresion_aritmetica v2 = expresion_aritmetica
	 	{res = Choco.eq(v1,v2);})
	 | #(DISTINTO v1 = expresion_aritmetica v2 = expresion_aritmetica
	 	{res = Choco.neq(v1,v2);});
 
expresion_aritmetica returns [IntegerExpressionVariable res = null;]
{IntegerExpressionVariable v1 = null, v2 = null;}: 
	   #(MAS v1 = expresion_aritmetica v2 = expresion_aritmetica
	   	{res = Choco.plus(v1,v2);})
	 | #(MENOS v1 = expresion_aritmetica v2 = expresion_aritmetica
	   	{res = Choco.minus(v1,v2);})
	 | #(MULT v1 = expresion_aritmetica v2 = expresion_aritmetica
	 	{res = Choco.mult(v1,v2);})
	 | #(DIV v1 = expresion_aritmetica v2 = expresion_aritmetica
	 	{res = Choco.div(v1,v2);})
	 | #(MOD v1 = expresion_aritmetica v2 = expresion_aritmetica
	 	{res = Choco.mod(v1,v2);})
	 | #(POW v1 = expresion_aritmetica v2 = expresion_aritmetica
	 	{res = Choco.power(v1,v2);})
	 | #(MENOS_UNARIO v1 = expresion_aritmetica)
	 | res = valor;
	 //| i:IDENT {res = getVariable(#i);};
	 
valor returns [IntegerExpressionVariable res = null;]: 
	  i:LIT_ENTERO {res = toInt(#i);} 
	| b:LIT_BOOLEANO {res = toBoolean(#b);}
	| j:LIT_STRING {res = toConstant(#j);}
	| k:IDENT {res = getVariable(#k);};